#!/usr/bin/lua

-- wbgen2, (c) 2010 Tomasz Wlostowski
--                  CERN BE-Co-HT
-- LICENSED UNDER GPL v2


------------------------------
-- HDL syntax tree constructors
------------------------------

-- assignment: dst <= src;
function va (dst, src)
  local s={};
  s.t="assign";
  s.dst=dst;
  s.src=src;
  return s;
end

-- index: name(h downto l)

function vi(name, h, l)
 local s={};
 s.t="index";
 s.name=name;
 s.h=h;
 s.l=l;
 return s;
end

-- synchronous process: process(clk, rst) begin {code} end process; 
function vsyncprocess(clk, rst, code)
 local s={};
 s.t="syncprocess";
 s.clk=clk;
 s.rst=rst;
 s.code=code;
 return s;
end

-- reset in process
function vreset(level, code)
 local s={};
 s.t="reset";
 s.level=level;
 s.code=code;
 return s;
end

function vposedge(code)
 local s={};
 s.t="posedge";
 s.code=code;
 return s;
end

function vif(cond, code, code_else)
 local s={};
 s.t="if";
 s.cond={ cond };
 s.code=code;
 s.code_else=code_else;
 return s;
end

function vequal(a,b)
 local s={};
 s.t="eq";
 s.a=a;
 s.b=b;
 return s;
end

function vand(a,b)
 local s={};
 s.t="and";
 s.a=a;
 s.b=b;
 return s;
end

function vnot(a)
 local s={};
 s.t="not";
 s.a=a;
 return s;
end

function vswitch(a, code)
 local s={};
 s.t="switch";
 s.a=a;
 s.code=code;
 return s;
end

function vcase(a, code)
 local s={};
 s.t="case";
 s.a=a;
 s.code=code;
 return s;
end

function vcasedefault(code)
 local s={};
 s.t="casedefault";
 s.code=code;
 return s;
end

function vcomment(str)
 local s={};
 s.t="comment";
 s.str=str;
 return s;
end

function vsub(a,b)
 local s={};
 s.t="sub";
 s.a=a;
 s.b=b;
 return s;
end



-- constructor for a HDL signal
function signal(type, nbits, name, comment)
    local t = {}
    t.comment = comment;
    t.type = type;
    t.range= nbits;
    t.name = name;
    return t;
end

-- constructor for a HDL port
function port(type, nbits, dir, name, comment)
    local t = {}
	t.comment = comment;
    t.type = type;
    t.range= nbits;
    t.name = name;
    t.dir = dir;
    return t;
end


function cgen_build_wishbone_ports()
    local ports = {
			port(BIT, 0, "in", "rst_n_i"),
			port(BIT, 0, "in", "wb_clk_i"),
		};
	
    if(address_bus_width > 0 ) then
			table_join(ports, { port(SLV, address_bus_width, "in", "wb_addr_i") });
    end

    table_join(ports, {
	port(SLV, DATA_BUS_WIDTH, "in", "wb_data_i"),
	port(SLV, DATA_BUS_WIDTH, "out", "wb_data_o"),
	port(BIT, 0, "in", "wb_cyc_i"),
	port(BIT, 0, "in", "wb_sel_i"),
	port(BIT, 0, "in", "wb_stb_i"),
	port(BIT, 0, "in", "wb_we_i"),
	port(BIT, 0, "out", "wb_ack_o")
    });
    
    return ports;
end


function cgen_build_clock_list()
    local allclocks = tree_2_table("clock");
    local i,v;
    local clockports = {};
    
    remove_duplicates(allclocks);
    
    for i,v in pairs(allclocks) do
    	table.insert(clockports, port(BIT, 0, "in", v));
    end

    return clockports;
end

function cgen_build_siglist()
	local siglist = {};
	local i,v;
	local s;
	
	siglist = tree_2_table("signals");
	
	wb_sigs = { 	    signal(BIT, 0, "wb_ack_regbank"),
								    signal(UNSIGNED, 4, "ack_cntr"),
								    signal(BIT,0, "ack_in_progress"),
								    signal(BIT,0, "tmpbit"),
								    signal(SLV, DATA_BUS_WIDTH, "rddata_reg"),
								    signal(SLV, DATA_BUS_WIDTH, "wrdata_reg"),
								    signal(SLV, address_bus_width, "rwaddr_reg")
 };
		
	table_join(siglist, wb_sigs);
	
	return siglist;
end



function cgen_build_portlist()
		local portlist = {};
    table_join(portlist, cgen_build_wishbone_ports());
    table_join(portlist, cgen_build_clock_list());
    table_join(portlist, tree_2_table("ports"));
		return portlist;
end

function cgen_find_sigport(name)
	for i,v in pairs(g_portlist) do if(name == v.name) then return v; end end
	for i,v in pairs(g_siglist) do if(name == v.name) then return v; end end
	return nil;
end

function cgen_build_signals_ports()
	g_portlist = cgen_build_portlist();
	g_siglist = cgen_build_siglist();
end

cur_indent = 0;

function indent_zero()
	cur_indent=0;
end

function indent_left()
	cur_indent = cur_indent - 1;
end

function indent_right()
	cur_indent = cur_indent + 1;
end


function cgen_new_snippet()
	emit_code = "";
end

function emiti()
	local i;
	for i = 1,cur_indent do emit_code=emit_code.."  "; end
end

function emit(s)
	local i;
	
	for i = 1,cur_indent do emit_code=emit_code.."  "; end
	emit_code=emit_code..s.."\n";
end

function emitx(s)
	emit_code=emit_code..s;
end

function cgen_get_snippet()
  return emit_code;
end

function cgen_write_current_snippet()
	hdl_file.write(hdl_file, emit_code);
end

function cgen_write_snippet(s)
	hdl_file.write(hdl_file, s);
end


function cgen_generate_hdl_init(filename)
	hdl_file = io.open(filename, "w");
end

function cgen_generate_hdl_done()
	hdl_file.close(hdl_file);
end
